@@ -331,6 +331,8 @@ typedef struct {
 #define QUARTER_PERCENTS 4
 #define TWELFTHS 12
 #define EIGHTHS 8
+#define TEACH_STRING0 "Enter calculation X+Y, X-Y, X*Y, X/Y, or Xv where X positive and result positive."
+#define TEACH_STRING1 "Press enter to go through calculation steps."
 
 typedef struct _AbacusRec *AbacusWidget;
 typedef struct _AbacusRec *AbacusDemoWidget;
