@@ -228,7 +228,8 @@ typedef struct _AbacusPart {
 	Pixmap      bufferBead[3][2][2];
 	TimeVal     oldTime;
 	Boolean     rightToLeftAdd, rightToLeftMult;
-	int         step, carry[2], lower, upper, carryStep, state;
+	int         step, carry[2], lower, upper, carryStep;
+	int         state; /* explain or carrying out operation */
 	int         reg, regCount, qDigit, qPosition;
 	double      divisor;
 	char        op;
