@@ -28,7 +28,22 @@
 #define EPSILON 0.0000001
 
 #ifdef UNDERCONSTRUCTION
-#define DEBUG
+#ifdef DEBUG
+#undef TEST
+#else
+#define TEST
+#endif
+#endif
+#ifdef KOJIMA_ROOT
+#undef LEE_ROOT
+#else
+#define LEE_ROOT
+#endif
+
+#ifdef TEST
+static void testTeach(AbacusWidget w, char op,
+	int startX, int finishX, int startY, int finishY,
+	int max);
 #endif
 
 /* ignores decimal point and multiply signs, etc, etc */
@@ -97,7 +112,7 @@ static void simpleParser(AbacusWidget w, char * buffer,
 
 /* More generic */
 #if 1
-/* Taken from addition tables in "How to Learn Lee's Abacus, */
+/* Taken from addition tables in "How to Learn Lee's Abacus", */
 /* made a little more generic, to handle other bases. */
 static void digitAdd(int a, int b, int carryIn,
 		int base, int topFactor, int bottomNumber,
@@ -130,8 +145,8 @@ static void digitAdd(int a, int b, int carryIn,
 #endif
 }
 
-/* Taken from subtraction tables in "How to Learn Lee's Abacus, */
-/* made a little more generic, go handle other bases. */
+/* Taken from subtraction tables in "How to Learn Lee's Abacus", */
+/* made a little more generic, to handle other bases. */
 static void digitSubtract(int a, int b, int carryIn,
 		int base, int topFactor, int bottomNumber,
 		int* lower, int* upper, int* carryOut)
@@ -164,8 +179,8 @@ static void digitSubtract(int a, int b, int carryIn,
 }
 
 #else
-/* Taken from addition tables in "How to Learn Lee's Abacus */
-/* less generic */
+/* Taken from addition tables in "How to Learn Lee's Abacus" */
+/* less generic, but easier to follow from book. */
 static void digitAdd(int a, int b, int carryIn,
 		int base, int topFactor, int bottomNumber,
 		int* lower, int* upper, int* carryOut)
@@ -219,8 +234,8 @@ static void digitAdd(int a, int b, int carryIn,
 #endif
 }
 
-/* Taken from subtraction tables in "How to Learn Lee's Abacus */
-/* less generic */
+/* Taken from subtraction tables in "How to Learn Lee's Abacus" */
+/* less generic but easier to follow from book. */
 static void digitSubtract(int a, int b, int carryIn,
 		int base, int topFactor, int bottomNumber,
 		int* lower, int* upper, int* carryOut)
@@ -379,17 +394,13 @@ static int getDecimalArrayPosition(int * array)
 }
 
 /* Appends decimal where necessary for consistency */
-static void decimalSafe(char * aString, char * bString)
+static void decimalSafe(char * string)
 {
-	int aDecimal = -1, bDecimal = -1;
+	int decimal = -1;
 
-	aDecimal = getDecimalStringPosition(aString);
-	if (aDecimal == (int) strlen(aString)) {
-		(void) strcat(aString, ".");
-	}
-	bDecimal = getDecimalStringPosition(bString);
-	if (bDecimal == (int) strlen(bString)) {
-		(void) strcat(bString, ".");
+	decimal = getDecimalStringPosition(string);
+	if (decimal == (int) strlen(string)) {
+		(void) strcat(string, ".");
 	}
 }
 
@@ -439,7 +450,7 @@ static int decimalOffsetArray(int * array, int place)
 static void contractStringBuffer(char * string)
 {
 	int offset = getDecimalStringPosition(string);
-	int i;
+	int i, length;
 
 	for (i = 0; i < offset - 1; i++) {
 		if (string[0] == '0')
@@ -447,21 +458,78 @@ static void contractStringBuffer(char * string)
 		else
 			break;
 	}
-	if (offset < (int) strlen(string))
-		for (i = (int) strlen(string) - 1; i > 1; i--) {
+	offset = getDecimalStringPosition(string);
+	length = (int) strlen(string);
+	if (offset < length)
+		for (i = length - 1; i > 1; i--) {
 			if (string[i] == '0')
 				string[i] = '\0';
 			else
 				break;
 		}
+	length = (int) strlen(string);
 	if (string[0] != '\0' && string[0] == '.') { /* normalize */
-		for (i = (int) strlen(string); i >= 0; i--) {
+		for (i = length; i >= 0; i--) {
 			string[i + 1] = string[i];
 		}
 		string[0] = '0';
 	}
 }
 
+static void  trimStringBuffer(char *string)
+{
+	int last;
+
+	contractStringBuffer(string);
+	last = strlen(string) - 1;
+	if (string[last] == '.')
+		string[last] = '\0';
+}
+
+static void stripDecimal(char * string) {
+	int i, found = 0, length;
+
+	for (i = 0; i < (int) strlen(string); i++) {
+		if (found == 1 || string[i] == '.') {
+			string[i] = string[i + 1];
+			found = 1;
+		}
+	}
+	found = -1;
+	for (i = 0; i < (int) strlen(string); i++)
+		if (string[i] != '0') {
+			found = i;
+			break;
+		}
+	if (found == -1) {
+		string[0] = '0';
+		string[1] = '\0';
+		return;
+	}
+	if (found == 0)
+		return;
+	length = strlen(string);
+	for (i = found; i < length; i++)
+		string[i - found] = string[i];
+	string[length - found] = '\0';
+}
+
+#ifdef LEE_ROOT
+/* Strip StringBuffer of trailing len numbers */
+static void stripStringBuffer(char *string, int len)
+{
+	int length;
+
+	trimStringBuffer(string);
+	stripDecimal(string);
+	if (len < 0) /* may have to revisit this */
+		return;
+	length = strlen(string);
+	if (length >= len)
+		string[length - len] = '\0';
+}
+#endif
+
 /* Expand StringBuffer to fit decimal place */
 static void expandStringBuffer(char * string, int place)
 {
@@ -470,6 +538,7 @@ static void expandStringBuffer(char * string, int place)
 	int appendOffset = offset + 1 - (int) strlen(string) - place;
 	int i, len;
 
+	decimalSafe(string);
 	len = (int) strlen(string);
 	if (place > 0 && prependOffset > 0) {
 		for (i = len; i >= 0; i--) {
@@ -580,22 +649,6 @@ stringGroup(int *intGroup, char *group, char v)
 	}
 }
 
-#ifdef UNDERCONSTRUCTION
-static int
-placeGroup(int *intGroup)
-{
-	int i = 0;
-
-	for (; *intGroup != -2; intGroup++) {
-		if (*intGroup == -1) {
-			return i;
-		}
-		i++;
-	}
-	return i;
-}
-#endif
-
 /* Set string given change in rail (decimal position) */
 static void setStringBuffer(AbacusWidget w, char * string, int aux,
 		int place, int lower, int upper)
@@ -619,20 +672,23 @@ static void setStringBuffer(AbacusWidget w, char * string, int aux,
 	offset = decimalOffsetString(string, place);
 	string[offset] = DIGIT_TO_CHAR((digit + wa->abacus.base +
 		CHAR_TO_DIGIT(string[offset])) % wa->abacus.base);
+#ifdef OLD_DEBUG
+	(void) printf("setStringBuffer %s %d\n",  string, place);
+#endif
 }
 
-/* Appends 0's where necessary to adding is easier */
+/* Appends 0's where necessary to make adding is easier */
 /* (before and after decimal point) */
 static void addSafe(char * aString, char * bString)
 {
 	int aDecimal = -1, bDecimal = -1;
 	int aCount, bCount, i;
+	char temp[512];
 
 	aDecimal = getDecimalStringPosition(aString);
 	bDecimal = getDecimalStringPosition(bString);
 	aCount = (int) strlen(aString) - aDecimal;
 	bCount = (int) strlen(bString) - bDecimal;
-	char temp[512];
 
 	if (aCount > bCount) {
 		for (i = bCount; i < aCount; i++)
@@ -746,6 +802,8 @@ static int nextCharPosition(AbacusWidget w, int step, int carryStep, int aux,
 	Boolean piece, piecePercent, subdeck;
 	AbacusWidget *wap = &w, wa;
 
+	if ((int) strlen(string) == getDecimalStringPosition(string))
+		count = strlen(string); /* no decimal point */
 	if (aux == 1 && w->abacus.leftAux != NULL)
 		wap = (AbacusWidget *) w->abacus.leftAux;
 	else if (aux == 2 && w->abacus.rightAux != NULL)
@@ -785,8 +843,9 @@ static int nextCharPosition(AbacusWidget w, int step, int carryStep, int aux,
 		else
 			a = n;
 	}
-#ifdef OLD_DEBUG
-	(void) printf("nextCharPosition: %d at step %d in %s\n", a, step, string);
+#ifdef DEBUG
+	(void) printf("nextCharPosition: %d at step %d in %s, n %d, decimal %d, count %d\n",
+		a, step, string, n, decimal, count);
 #endif
 	return a;
 }
@@ -807,6 +866,92 @@ static int nextChar(char * string, int pos)
 	return digit;
 }
 
+#ifdef LEE_ROOT
+static int placeGroup(int *intGroup)
+{
+	int i = 0;
+
+	for (; *intGroup != -2; intGroup++) {
+		if (*intGroup == -1) {
+			return i;
+		}
+		i++;
+	}
+	return i;
+}
+
+/* int value of string offset by place */
+static int andAbove(char * string, int place, int base)
+{
+	int i, value = 0, factor = 1, pos;
+	int runover = decimalOffsetString(string, place) - strlen(string);
+	int newPlace = place;
+
+	if (runover > 0)
+		newPlace += runover;
+	for (i = 0; i < (int) strlen(string); i++) {
+		pos = decimalOffsetString(string, newPlace + i);
+		if (pos < 0)
+			break;
+		value += nextChar(string, pos) * factor;
+		factor *= base;
+	}
+	for (i = 0; i < runover; i++) {
+		value *= base;
+	}
+#ifdef DEBUG
+	(void) printf("andAbove: %d at %d in %s, runover%d\n",
+		value, place, string, runover);
+#endif
+	return value;
+}
+
+/* number of places where int value of string offset by place */
+static int andAbovePlaces(char * string, int place)
+{
+	int i, pos;
+	int runover = decimalOffsetString(string, place) - strlen(string);
+	int newPlace = place;
+
+	if (runover > 0)
+		newPlace += runover;
+	for (i = 0; i < (int) strlen(string); i++) {
+		pos = decimalOffsetString(string, newPlace + i);
+		if (pos < 0) {
+			break;
+		}
+	}
+	if (runover > 0)
+		i += runover;
+#ifdef DEBUG
+	(void) printf("andAbovePlaces: %d at %d in %s, pos %d\n", i, place, string, pos);
+#endif
+	return i;
+}
+
+#if 0
+static int digitInPlace(int value, int pos, int base)
+{
+	int i;
+
+	for (i = 0; i < pos; i++)
+		value /= base;
+	return value % base;
+}
+#endif
+
+static int logInt(int value, int base)
+{
+	int count = 0;
+
+	while (value >= base) {
+		 value /= base;
+		count++;
+	}
+	return count;
+}
+#endif
+
 /* Number of steps in addition and subtraction */
 static int addSteps(char * string)
 {
@@ -943,7 +1088,7 @@ static void rootGroup(AbacusWidget w, char* string, int root, int** group)
 }
 
 #ifdef DEBUG
-#ifdef KOJIMA
+#ifdef KOJIMA_ROOT
 /* Implement an iteration of Newton's Method or Halley's Method */
 static double rootAdvance(double x, double p, int n)
 {
@@ -1032,7 +1177,7 @@ static int intGroupValue(AbacusWidget w, int position) {
 	if (position >= arrayLen(w->abacus.intGroup))
 		return 0;
 	for (i = 0; i <= position; i++) {
-		if (w->abacus.intGroup[i] == -1) { // only one decimal point
+		if (w->abacus.intGroup[i] == -1) { /* only one decimal point */
 			newPosition = position + 1;
 			break;
 		}
@@ -1061,15 +1206,17 @@ static int testSqrtAdvance(AbacusWidget w, int power,
 {
 	int digitAdd = 1;
 	int i = 0;
+#if 0
 	digitStruct *ds;
 
 	ds = (digitStruct *) calloc(w->abacus.base - 1, sizeof (digitStruct));
+#endif
 #ifdef DEBUG
 	(void) printf("remainder power = %d\n", power);
 #endif
 	while (*groupSum + *groupInc <= power) {
 		*groupInc += digitAdd;
-#if UNDERCONSTRUCTION
+#if 0
 		ds[i].digitAdd = digitAdd;
 		ds[i].groupInc = *groupInc;
 #endif
@@ -1086,7 +1233,9 @@ static int testSqrtAdvance(AbacusWidget w, int power,
 	(void) printf("sum %d: %d %d : %d\n",
 		i, *digitSum, *groupInc, *groupSum);
 #endif
+#if 0
 	free(ds);
+#endif
 	return i;
 }
 
@@ -1127,7 +1276,7 @@ static void testSqrt(AbacusWidget w, int position)
 		allDigitSum = allDigitSum * w->abacus.base + digitSum;
 	}
 	allDigitSum--;
-	sqrt = (allDigitSum + root - 1) / root;
+	sqrt = (double) (allDigitSum + root - 1) / root;
 #ifdef DEBUG
 	(void) printf("allDigitSum %d, sqrtm = %g\n",
 		allDigitSum, sqrt);
@@ -1227,7 +1376,7 @@ static void testCbrt(AbacusWidget w, int position)
 		allDigitSum = allDigitSum * w->abacus.base + digitSum;
 	}
 	allDigitSum--;
-	cbrt = (allDigitSum + root - 1) / root;
+	cbrt = (double) (allDigitSum + root - 1) / root;
 #ifdef DEBUG
 	(void) printf("allDigitSum %d, cbrtm = %g\n",
 		allDigitSum, cbrt);
@@ -1256,7 +1405,9 @@ highlightRail(AbacusWidget w, int aux, int rail, Boolean highlight)
 		wap = (AbacusWidget *) w->abacus.rightAux;
 	}
 	wa = *wap;
+#ifndef TEST
 	drawBeadRail(wa, rail + wa->abacus.decimalPosition, highlight);
+#endif
 }
 
 /* Tell about what is going to happen */
@@ -1307,7 +1458,7 @@ static Boolean nextPositionSum(AbacusWidget w, char op,
 		int *rPosition, int *rDigit, int *bDigit)
 {
 	int n = (w->abacus.step - 2) / 2; /* 2 step display */
-	int max = addSteps(w->abacus.aString);
+	int max = addSteps(w->abacus.aString); /* number of steps with original */
 	int topFactor = w->abacus.decks[TOP].factor;
 	int bottomNumber = w->abacus.decks[BOTTOM].number;
 	Boolean rightToLeft = w->abacus.rightToLeftAdd;
@@ -1371,8 +1522,8 @@ static Boolean nextPositionProduct(AbacusWidget w,
 		w->abacus.bString);
 	bPlace = decimalPlaceString(w->abacus.bString, bOffset);
 	*bDigit = nextChar(w->abacus.bString, bOffset);
-	rPlace = ((n % 2 == 0 && !rightToLeft) ||
-		(n % 2 == 1 && rightToLeft)) ? 1 : 0;
+	rPlace = (((n & 1) == 0 && !rightToLeft) ||
+		((n & 1) == 1 && rightToLeft)) ? 1 : 0;
 	if (w->abacus.carryStep != 0) {
 		rPlace += w->abacus.carryStep / 2;
 		*bDigit = 1;
@@ -1383,8 +1534,8 @@ static Boolean nextPositionProduct(AbacusWidget w,
 	rDigit = nextChar(w->abacus.rString, rOffset);
 	if (w->abacus.carryStep == 0) {
 		/* 2 digits * 2 step display */
-		if (((n % 2 == 0) && rightToLeft) ||
-				((n % 2 == 1) && !rightToLeft)) {
+		if ((((n & 1) == 0) && rightToLeft) ||
+				(((n & 1) == 1) && !rightToLeft)) {
 			*bValue = (*aDigit * *bDigit) % w->abacus.base;
 		} else {
 			*bValue = (*aDigit * *bDigit) / w->abacus.base;
@@ -1402,7 +1553,7 @@ static Boolean nextPositionProduct(AbacusWidget w,
 		&(w->abacus.lower), &(w->abacus.upper),
 		&(w->abacus.carry[w->abacus.state]));
 	*rPosition = rPlace;
-	if (w->abacus.carry[w->abacus.state] != 0 && n % 2 == 1 &&
+	if (w->abacus.carry[w->abacus.state] != 0 && ((n & 1) == 1) &&
 			rightToLeft)
 		return True; /* Do not want to forget carry */
 	return (n >= max - 1);
@@ -1478,8 +1629,8 @@ static Boolean nextPositionDivision(AbacusWidget w,
 		w->abacus.bString);
 	bPlace = decimalPlaceString(w->abacus.bString, bOffset);
 	*bDigit = nextChar(w->abacus.bString, bOffset);
-	rPlace = ((n % 2 == 0 && !rightToLeft) ||
-		(n % 2 == 1 && rightToLeft)) ? 1 : 0;
+	rPlace = ((((n & 1) == 0) && !rightToLeft) ||
+		(((n & 1) == 1) && rightToLeft)) ? 1 : 0;
 	if (w->abacus.carryStep != 0) {
 		rPlace += w->abacus.carryStep / 2;
 		*bDigit = 1;
@@ -1494,8 +1645,8 @@ static Boolean nextPositionDivision(AbacusWidget w,
 #endif
 	if (w->abacus.carryStep == 0) {
 		/* 2 digits * 2 step display */
-		if (((n % 2 == 0) && rightToLeft) ||
-				((n % 2 == 1) && !rightToLeft)) {
+		if ((((n & 1) == 0) && rightToLeft) ||
+				(((n & 1) == 1) && !rightToLeft)) {
 			*bValue = (w->abacus.qDigit * *bDigit) % w->abacus.base;
 		} else {
 			*bValue = (w->abacus.qDigit * *bDigit) / w->abacus.base;
@@ -1515,7 +1666,7 @@ static Boolean nextPositionDivision(AbacusWidget w,
 	return (n >= 2 * max + 2 * w->abacus.decimalPosition - 1);
 }
 
-/* Handle root one step at a time */
+/* Handle root one step at a time, Kojima */
 static Boolean nextPositionRoot(AbacusWidget w, int root,
 		int *rPosition, int *aDigit, int *bDigit, int *bValue)
 {
@@ -1553,7 +1704,7 @@ static Boolean nextPositionRoot(AbacusWidget w, int root,
 #endif
 		w->abacus.qPosition = aPlace;
 #ifdef DEBUG
-#ifdef KOJIMA
+#ifdef KOJIMA_ROOT
 		testRoot(w, w->abacus.qDigit, aPlace, root);
 #endif
 		if (root == 2)
@@ -1581,50 +1732,359 @@ static Boolean nextPositionRoot(AbacusWidget w, int root,
 	return True;
 }
 
-#ifdef UNDERCONSTRUCTION
+#ifdef LEE_ROOT
+/* Handle Auxiliary Operations Field table one step at a time */
+static Boolean nextPositionAOField(AbacusWidget w,
+		int *rPosition, int *bDigit, int count)
+{
+	int topFactor = w->abacus.decks[TOP].factor;
+	int bottomNumber = w->abacus.decks[BOTTOM].number;
+	Boolean rightToLeft = w->abacus.rightToLeftAdd; /* only adding on 1 digit */
+	int aOFPos, aOFDigit;
+
+	if (w->abacus.leftAux != NULL) {
+		AbacusWidget *wap = &w, wa;
+
+		wap = (AbacusWidget *) w->abacus.leftAux;
+		wa = *wap;
+		topFactor = wa->abacus.decks[TOP].factor;
+		bottomNumber = wa->abacus.decks[BOTTOM].number;
+	}
+	*rPosition = w->abacus.regCount;
+	if (w->abacus.carryStep != 0)
+		*rPosition += w->abacus.carryStep / 2;
+#ifdef DEBUG
+printf("AOF carry:%d   cs%d\n", *rPosition, w->abacus.carryStep);
+#endif
+	aOFPos = decimalOffsetString(w->abacus.bString, *rPosition);
+	aOFDigit = nextChar(w->abacus.bString, aOFPos);
+	if (w->abacus.carryStep == 0) {
+		*bDigit = 1 + ((count > 0) ? 1 : 0);
+	} else {
+		*bDigit = 1;
+	}
+	if (!rightToLeft)
+		w->abacus.carry[w->abacus.state] = 0;
+	digitAdd(aOFDigit, *bDigit, w->abacus.carry[w->abacus.state],
+		w->abacus.base, topFactor, bottomNumber,
+		&(w->abacus.lower), &(w->abacus.upper),
+		&(w->abacus.carry[w->abacus.state]));
+#ifdef DEBUG
+	(void) printf("AOF: bs %s, bDigit %d, aOFPos %d, aOFDigit %d, rPosition %d\n",
+w->abacus.bString, *bDigit, aOFPos, aOFDigit, *rPosition);
+#endif
+	return True; /* FIXME? */
+}
+
+/* Handle Primary Operations Field table one step at a time */
+static Boolean nextPositionPOField(AbacusWidget w,
+		int *rPosition, int *bDigit, int count)
+{
+	int topFactor = w->abacus.decks[TOP].factor;
+	int bottomNumber = w->abacus.decks[BOTTOM].number;
+	Boolean rightToLeft = w->abacus.rightToLeftAdd;
+	int pOFPos, pOFDigit;
+	int root = (w->abacus.op == 'v') ? 2 : 3;
+	int cPos, place;
+
+	/* cString is bString or shorter */
+	(void) sprintf(w->abacus.cString, "%s", w->abacus.bString);
+	stripStringBuffer(w->abacus.cString, w->abacus.regCount);
+	*rPosition = w->abacus.regCount * root;
+        cPos = nextCharPosition(w, count, w->abacus.carryStep, 0,
+                rightToLeft, w->abacus.cString);
+	place = decimalPlaceString(w->abacus.cString, cPos);
+	*rPosition += place; 
+	if (w->abacus.carryStep == 0) {
+		*bDigit = nextChar(w->abacus.cString, cPos);
+#ifdef DEBUG
+(void) printf("DAB: bDigit%d, place%d, reg%d, regCount%d cPos%d\n",
+  *bDigit, place, w->abacus.reg, w->abacus.regCount, cPos);
+#endif
+	} else {
+		*bDigit = 1;
+#ifdef DEBUG
+(void) printf("DAB: carry bDigit%d, place%d carryStep%d\n",
+  *bDigit, place, w->abacus.carryStep / 2);
+#endif
+	}
+	/* rString can expand with carries. */
+	pOFPos = decimalOffsetString(w->abacus.rString, *rPosition);
+	pOFDigit = nextChar(w->abacus.rString, pOFPos);
+	if (!rightToLeft)
+		w->abacus.carry[w->abacus.state] = 0;
+	digitSubtract(pOFDigit, *bDigit, w->abacus.carry[w->abacus.state],
+		w->abacus.base, topFactor, bottomNumber,
+		&(w->abacus.lower), &(w->abacus.upper),
+		&(w->abacus.carry[w->abacus.state]));
+#ifdef DEBUG
+	(void) printf("POF: rs %s, cs %s, pOFPos %d, pOFDigit %d, rPosition %d, bDigit %d, count %d, place %d, carry %d\n",
+		w->abacus.rString, w->abacus.cString, pOFPos, pOFDigit, *rPosition, *bDigit, count, place, w->abacus.carry[w->abacus.state]);
+#endif
+	return True; /* FIXME? */
+}
+
 /* A little tricky as this is reentrant */
-void teachStepRoot(AbacusWidget w, char * buffer)
+/* Works for 625v, 166464v, 1004004v, 10201v and squares 0-20 */
+/* Eliminate adding and subtracting 0 */
+/* Need floating point */
+void teachStepRoot(AbacusWidget w)
 {
-	char buffer1[120] = "";
 	char buffer2[120] = "";
 	char buffer3[120] = "";
-	int base = w->abacus.base;
-	int bottomNumber = w->abacus.decks[BOTTOM].number;
-	double a, b;
+	Boolean done = False;
+	int bDigit = 0, rValue, bValue;
+	int rPosition = 0, rPlace;
+	int rStep = w->abacus.step - 2;
+	int nsteps; /* number of steps in interation */
 	int root = (w->abacus.op == 'v') ? 2 : 3;
-	int place;
+	double r = convertToDecimal(w->abacus.base, w->abacus.rString);
+	double a = convertToDecimal(w->abacus.base, w->abacus.aString);
+	Boolean almostDone = (r == 0.0 && r < a);
+	int futureOffset = 1;
+	int decimalPosition = w->abacus.decimalPosition;
+	int decimalPosition1 = w->abacus.decimalPosition;
 
-	if (w->abacus.step == 2) {
-		place = placeGroup(w->abacus.intGroup) - 1;
-		(void) sprintf(buffer2,
-			"Add %d to left A-O field, offset digit by corresponding group %d",
-			1, place);
+	if (w->abacus.leftAux != NULL) {
+		AbacusWidget *wap = &w, wa;
+
+		wap = (AbacusWidget *) w->abacus.leftAux;
+		wa = *wap;
+		decimalPosition1 = wa->abacus.decimalPosition;
+	}
+	if (w->abacus.reg == -1) {
+		w->abacus.regCount = placeGroup(w->abacus.intGroup) - 1;
+		w->abacus.reg = 0;
+	}
+	rValue = andAbove(w->abacus.rString, w->abacus.regCount * root,
+		w->abacus.base);
+	bValue = andAbove(w->abacus.bString, w->abacus.regCount,
+		w->abacus.base);
+	rPlace = andAbovePlaces(w->abacus.rString, w->abacus.regCount * root);
+	nsteps = 4 + 2 * rPlace;
+	if (rStep / nsteps == 0) {
+		futureOffset = 0;
+	}
+	if (bValue + futureOffset >= rValue || w->abacus.reg == 1) {
+		almostDone = True;
+		w->abacus.reg = 1; /* need for carry as bValue changes */
+	}
+#ifdef DEBUG
+(void) printf("DAB: rV%d bV%d, rP%d reg%d, regCount%d   %d    %d %d %g %g\n",
+	rValue, bValue, rPlace, w->abacus.reg, w->abacus.regCount, almostDone, rStep, nsteps, r, a);
+#endif
+	if (rStep % nsteps == 0) {
+		if (almostDone) {
+			if (futureOffset == 0) {
+				/* Have not started with this position and
+				   nothing here */
+				w->abacus.step = 2;
+				w->abacus.regCount--;
+				w->abacus.reg = 0;
+				if (w->abacus.regCount < -decimalPosition1 ||
+						w->abacus.regCount < -decimalPosition / root) {
+					w->abacus.step = 0;
+					(void) sprintf(buffer3, "Answer (divide by 2): %s",
+						w->abacus.bString);
+					drawLineText(w, buffer3, 2);
+					(void) sprintf(buffer2, "done");
+				} else {
+					(void) sprintf(buffer2, "Now try smaller value");
+				}
+			} else {
+				(void) sprintf(buffer2,
+					"Almost done with position, need to add 1 to place working on");
+				w->abacus.step += 2;
+			}
+			drawLineText(w, buffer2, 1);
+		} else {
+			(void) sprintf(buffer2,
+				"Left A-O field value %d%s < P-O field group value %d",
+				bValue, ((futureOffset != 0) ? " + 1" : ""),
+				rValue);
+			drawLineText(w, buffer2, 1);
+			w->abacus.step++;
+		}
+	} else if (rStep % nsteps == 1) {
+		if (bValue + futureOffset < rValue) {
+			(void) sprintf(buffer2, "Yes, ok to iterate");
+			w->abacus.step++;
+		} else {
+			if (rValue == 0) {
+				w->abacus.step = 0;
+				(void) sprintf(buffer3, "Answer (divide by %d): %s",
+					root, w->abacus.bString);
+				drawLineText(w, buffer3, 2);
+				(void) sprintf(buffer2, "Done");
+			} else {
+				(void) sprintf(buffer2, "Try smaller value");
+				w->abacus.step++;
+			}
+		}
 		drawLineText(w, buffer2, 1);
-		highlightRail(w, 1, place, True);
-		w->abacus.step++;
-	} else if (w->abacus.step == 3) {
-		/* a carry should never be needed */
-		place = placeGroup(w->abacus.intGroup) - 1;
-		setAbacusMove(w, ACTION_MOVE, 1, 0, place, 1);
-		highlightRail(w, 1, place, False);
-		w->abacus.step++;
-	} else if (w->abacus.step == 4) {
-		place = placeGroup(w->abacus.intGroup) - 1;
-		(void) sprintf(buffer2,
-			"Subtract %d from primary field, offset digit by corresponding group %d",
-			1, place);
+		w->abacus.carryStep = 0;
+		w->abacus.carry[0] = 0;
+		w->abacus.carry[1] = 0;
+	} else if (rStep % nsteps == 2 ||
+			(rStep % nsteps == 3 &&
+			w->abacus.carryStep != 0 &&
+			w->abacus.carryStep % 2 == 0)) {
+		w->abacus.state = 0;
+		done = nextPositionAOField(w, &rPosition, &bDigit,
+			((almostDone) ? 0 : rStep / nsteps));
+		highlightRail(w, 1, rPosition, True);
+		if (w->abacus.carryStep != 0)
+			(void) sprintf(buffer2,
+				"Carrying, add %d to left A-O field, offset digit at position %d",
+				bDigit, rPosition); /* should only need carry once */
+		else
+			(void) sprintf(buffer2,
+				"Add %d to left A-O field, offset digit by corresponding group (or position) %d%s",
+				bDigit, rPosition,
+				((w->abacus.carry[w->abacus.state] != 0) ? ", with carry" : ""));
 		drawLineText(w, buffer2, 1);
-		highlightRail(w, 0, place * root, True);
-		w->abacus.step++;
-	} else if (w->abacus.step == 5) {
-		printf("do P\n");
-		place = (placeGroup(w->abacus.intGroup) - 1) * root;
-		setAbacusMove(w, ACTION_MOVE, 0, 0, place, -1);/*lower 6->5 */
-		highlightRail(w, 0, place, False);
-		w->abacus.step++;
-	} else if (w->abacus.step >= 6) {
-		printf("loop\n");
-		w->abacus.step++;
+		if (w->abacus.carryStep == 0) {
+			w->abacus.step++;
+		} else {
+			w->abacus.carryStep++;
+		}
+	} else if (rStep % nsteps == 3) {
+		 if (!w->abacus.rightToLeftAdd)
+			w->abacus.carry[1] = w->abacus.carry[0];
+		w->abacus.state = 1;
+		done = nextPositionAOField(w, &rPosition, &bDigit,
+			((almostDone) ? 0 : rStep / nsteps));
+		setStringBuffer(w, w->abacus.bString, 1,
+			rPosition, w->abacus.lower, w->abacus.upper);
+		if (w->abacus.lower != 0)
+			setAbacusMove(w, ACTION_MOVE, 1, 0, rPosition,
+				w->abacus.lower);
+		if (w->abacus.upper != 0)
+			setAbacusMove(w, ACTION_MOVE, 1, 1, rPosition,
+				w->abacus.upper);
+		highlightRail(w, 1, rPosition, False);
+		if (w->abacus.carry[w->abacus.state] == 0 &&
+				w->abacus.carryStep != 0) {
+			w->abacus.carryStep = 0;
+		}
+		contractStringBuffer(w->abacus.bString);
+		if (almostDone) {
+			if (w->abacus.carry[w->abacus.state] != 0) {
+				if (w->abacus.carryStep == 0) {
+					w->abacus.carryStep = 2;
+					if (w->abacus.rightToLeftAdd)
+						w->abacus.carry[1] = w->abacus.carry[0] = 0;
+				} else {
+					w->abacus.carryStep++;
+				}
+				(void) sprintf(buffer3, "Current answer (divide by %d): %s",
+					root, w->abacus.bString);
+				drawLineText(w, buffer3, 2);
+			} else {
+				w->abacus.step = 2;
+				w->abacus.reg = 0;
+				w->abacus.regCount--;
+				if (w->abacus.regCount < -decimalPosition1 ||
+						w->abacus.regCount < -decimalPosition / root) {
+					w->abacus.step = 0;
+					(void) sprintf(buffer3, "Answer (divide by %d): %s",
+						root, w->abacus.bString);
+					drawLineText(w, buffer3, 2);
+				} else {
+					(void) sprintf(buffer3, "Current answer (divide by %d): %s",
+						root, w->abacus.bString);
+					drawLineText(w, buffer3, 2);
+				}
+			}
+		} else {
+			(void) sprintf(buffer3, "Current answer (divide by %d): %s",
+				root, w->abacus.bString);
+			drawLineText(w, buffer3, 2);
+			if (w->abacus.carry[w->abacus.state] != 0) {
+				if (w->abacus.carryStep == 0) {
+					w->abacus.carryStep = 2;
+					if (w->abacus.rightToLeftAdd)
+						w->abacus.carry[1] = w->abacus.carry[0] = 0;
+				} else {
+					w->abacus.carryStep++;
+				}
+			} else {
+				w->abacus.carryStep = 0;
+			}
+			if (w->abacus.carryStep == 0) {
+				w->abacus.step++;
+			}
+		}
+	} else if (rStep % 2 == 0 || (w->abacus.carryStep != 0 &&
+			w->abacus.carryStep % 2 == 0)) {
+		w->abacus.state = 0;
+		done = nextPositionPOField(w, &rPosition, &bDigit,
+			(rStep % nsteps - 4) / 2);
+		highlightRail(w, 0, rPosition, True);
+		if (w->abacus.carryStep != 0) {
+			(void) sprintf(buffer2, "Borrowing");
+		} else {
+			if (bDigit != 0 && bDigit != bValue)
+				(void) sprintf(buffer2,
+					"Subtract %d (part of %d) from primary field, offset digit by corresponding position %d (in group %d)%s",
+					bDigit, bValue, rPosition, w->abacus.regCount,
+					((w->abacus.carry[w->abacus.state] != 0) ? ", with borrow" : ""));
+			else
+				(void) sprintf(buffer2,
+					"Subtract %d from primary field, offset digit by corresponding position %d (in group %d)%s",
+					bDigit, rPosition, w->abacus.regCount,
+					((w->abacus.carry[w->abacus.state] != 0) ? ", with borrow" : ""));
+		}
+		drawLineText(w, buffer2, 1);
+		if (w->abacus.carryStep == 0) {
+			w->abacus.step++;
+		} else {
+			w->abacus.carryStep++;
+		}
+	} else {
+		if (!w->abacus.rightToLeftAdd)
+			w->abacus.carry[1] = w->abacus.carry[0];
+		w->abacus.state = 1;
+		done = nextPositionPOField(w, &rPosition, &bDigit,
+			(rStep % nsteps - 4) / 2);
+		setStringBuffer(w, w->abacus.rString, 0,
+			rPosition, w->abacus.lower, w->abacus.upper);
+		if (w->abacus.lower != 0)
+			setAbacusMove(w, ACTION_MOVE, 0, 0, rPosition,
+				w->abacus.lower);
+		if (w->abacus.upper != 0)
+			setAbacusMove(w, ACTION_MOVE, 0, 1, rPosition,
+				w->abacus.upper);
+		highlightRail(w, 0, rPosition, False);
+		if (w->abacus.carry[w->abacus.state] == 0 &&
+				w->abacus.carryStep != 0) {
+			w->abacus.carryStep = 0;
+		}
+		if (rStep % nsteps != nsteps - 1 &&
+				w->abacus.carry[w->abacus.state] == 0 &&
+				logInt(bValue, w->abacus.base) <= (rStep % nsteps - 4) / 2) {
+			w->abacus.step = (w->abacus.step / nsteps + 1) * nsteps + 2;
+		} else {
+			if (!w->abacus.rightToLeftAdd) {
+				if (w->abacus.carry[w->abacus.state] != 0) {
+					if (w->abacus.carryStep == 0) {
+						w->abacus.carryStep = 2;
+						if (w->abacus.rightToLeftAdd)
+							w->abacus.carry[1] = w->abacus.carry[0] = 0;
+					} else {
+						w->abacus.carryStep++;
+					}
+				}
+				if (w->abacus.carryStep == 0) {
+					w->abacus.step++;
+				}
+			} else if (w->abacus.rightToLeftAdd) {
+				w->abacus.step++;
+                        }
+#ifdef DEBUG
+printf("DAB: THere %d\n", w->abacus.step);
+#endif
+		}
 	}
 }
 #endif
@@ -1639,12 +2099,25 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 	int bottomNumber = w->abacus.decks[BOTTOM].number;
 	double a, b;
 
+#ifdef TEST
+	if (aux == 0) {
+#if 0
+		testTeach(w, '+', 3, 8, 2, 3, 10);
+		testTeach(w, '+', 3, 8, 2, 3, 10);
+		testTeach(w, '+', 3, 8, 2, 3, 10);
+		testTeach(w, 'v', 1, 10102, 2, 3, 10000);
+#endif
+		testTeach(w, 'v', 0, 10102, 2, 3, 10000);
+	} else {
+		aux = 0;
+	}
+#endif
 	if (w->abacus.step == 0) {
 #ifdef OLD_DEBUG
 		testTable();
 #endif
-		drawLineText(w, (char *) "Enter calculation X+Y, X-Y, X*Y, or X/Y where X positive and result positive.", 0);
-		drawLineText(w, (char *) "Press enter to go through calculation steps.", 1);
+		drawLineText(w, (char *) TEACH_STRING0, 0);
+		drawLineText(w, (char *) TEACH_STRING1, 1);
 		(void) sprintf(buffer3, "(addition order will be: %s; multiplicand order will be %s)",
 			((w->abacus.rightToLeftAdd) ? "right to left" : "left to right"),
 			((w->abacus.rightToLeftMult) ? "right to left" : "left to right"));
@@ -1716,7 +2189,7 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			b = convertToDecimal(base, w->abacus.bString);
 			if (w->abacus.op == '-' && b > a) {
 				/* Revisit this, it should be allowed, but
-				   rails probably need to be respected. */
+				   # of rails probably need to be respected. */
 				(void) sprintf(buffer1,
 					"Subtraction underflow %s - %s",
 					w->abacus.aString, w->abacus.bString);
@@ -1729,7 +2202,8 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				w->abacus.aString, w->abacus.op,
 				w->abacus.bString);
 			drawLineText(w, buffer1, 0);
-			decimalSafe(w->abacus.aString, w->abacus.bString);
+			decimalSafe(w->abacus.aString);
+			decimalSafe(w->abacus.bString);
 			addSafe(w->abacus.aString, w->abacus.bString);
 			(void) strcpy(w->abacus.rString, w->abacus.aString);
 			(void) sprintf(buffer3, "Current answer: %s",
@@ -1757,12 +2231,15 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			(void) sprintf(buffer1, "Multiplying %s %c %s",
 				w->abacus.aString, w->abacus.op,
 				w->abacus.bString);
+#ifndef WINVER
 			if (!w->abacus.lee) {
 				(void) strcat(buffer1,
 					" ... works best with lee option");
 			}
+#endif
 			drawLineText(w, buffer1, 0);
-			decimalSafe(w->abacus.aString, w->abacus.bString);
+			decimalSafe(w->abacus.aString);
+			decimalSafe(w->abacus.bString);
 			(void) sprintf(w->abacus.rString, "0.");
 			(void) sprintf(buffer3, "Current answer: %s",
 				w->abacus.rString);
@@ -1796,12 +2273,15 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			(void) sprintf(buffer1, "Dividing %s %c %s",
 				w->abacus.aString, w->abacus.op,
 				w->abacus.bString);
+#ifndef WINVER
 			if (!w->abacus.lee) {
 				(void) strcat(buffer1,
 					" ... works best with lee option");
 			}
+#endif
 			drawLineText(w, buffer1, 0);
-			decimalSafe(w->abacus.aString, w->abacus.bString); /* quotient */
+			decimalSafe(w->abacus.aString); /* quotient */ 
+			decimalSafe(w->abacus.bString);
 			(void) strcpy(w->abacus.rString, w->abacus.aString);
 			(void) sprintf(w->abacus.sString, "0.");
 			(void) sprintf(buffer3, "Current answer: %s",
@@ -1827,10 +2307,12 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			(void) sprintf(buffer1, "%s root of %s, grouping digits yields (%s)",
 				((w->abacus.op == 'v') ? "Square" : "Cube"),
 				w->abacus.aString, group);
+#ifndef WINVER
 			if (!w->abacus.lee) {
 				(void) strcat(buffer1,
 					" ... works best with lee option");
 			}
+#endif
 			drawLineText(w, buffer1, 0);
 			(void) strcpy(w->abacus.rString, w->abacus.aString);
 			(void) sprintf(w->abacus.sString, "0.");
@@ -1838,8 +2320,11 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				w->abacus.sString);
 			drawLineText(w, buffer3, 2);
 			w->abacus.step++;
-			w->abacus.reg = 0;
+			w->abacus.reg = -1;
 			(void) strcpy (w->abacus.cString, w->abacus.aString); /* remainder */
+#ifdef LEE_ROOT
+			(void) sprintf(w->abacus.bString, "0.");
+#endif
 		}
 #ifdef DEBUG
 		(void) printf("op buffer: %s %c %s %d %d\n",
@@ -1849,10 +2334,10 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 #endif
 		w->abacus.carry[0] = 0;
 		w->abacus.carry[1] = 0;
-#ifdef UNDERCONSTRUCTION
+#ifdef LEE_ROOT
 	} else if (w->abacus.op == 'v' || w->abacus.op == 'u') {
 		/* Keep separate as its too different */
-		teachStepRoot(w, buffer);
+		teachStepRoot(w);
 		return;
 #endif
 	} else if (w->abacus.reg == 0 || (w->abacus.reg < 0 &&
@@ -1884,7 +2369,10 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				(void) strAddChar(buffer1, w->abacus.op);
 				(void) strcat(buffer1, " ");
 				(void) strcat(buffer1, buf);
-				convertFromInteger(buf, base, aDigit + bDigit);
+				if (w->abacus.op == '+')
+					convertFromInteger(buf, base, aDigit + bDigit);
+				else
+					convertFromInteger(buf, base, aDigit - bDigit);
 				(void) strcat(buffer1, " = ");
 				(void) strcat(buffer1, buf);
 				convertFromInteger(buf, base, bValue);
@@ -2024,25 +2512,6 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				(void) strcat(buffer1, " ... borrowing 1");
 			}
 			drawLineText(w, buffer1, 0);
-#ifdef LEE_ROOT
-		} else if (w->abacus.op == 'v' || w->abacus.op == 'u') {
-			char group[512];
-
-			group[0] = '\0';
-			done = nextPositionRoot(w,
-				(w->abacus.op == 'v') ? 2 : 3,
-				&rPosition, &aDigit, &bDigit, &bValue);
-			stringGroup(w->abacus.intGroup, group, w->abacus.op);
-			(void) sprintf(buffer1, "Taking %s Root of (%s)",
-				(w->abacus.op == 'v') ? "Square" : "Cube", group);
-				/*w->abacus.cString */
-			(void) strcat(buffer1, " , register ");
-			(void) strAddInt(buffer1, w->abacus.intGroup[0]);
-			(void) strcat(buffer1, " on primary, rail ");
-			drawLineText(w, buffer1, 0);
-			(void) pendingUpdate(w, buffer2, 1,
-				1, w->abacus.intGroup[0], w->abacus.decks[BOTTOM].number);
-#endif
 		}
 		if (!pendingUpdate(w, buffer2, 1, 0, rPosition, bottomNumber))
 			done = False;
@@ -2199,11 +2668,80 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 						w->abacus.carryStep++;
 					}
 				}
-				if (w->abacus.carryStep == 0)
+				if (w->abacus.carryStep == 0) {
 					w->abacus.step++;
+				}
 			} else if (w->abacus.rightToLeftAdd) {
 				w->abacus.step++;
 			}
 		}
 	}
 }
+
+#ifdef TEST
+void testTeach(AbacusWidget w, char op,
+	int startX, int finishX, int startY, int finishY,
+	int max)
+{
+	char buffer[120] = "";
+	int i, moves, x, y;
+	double ans, rem, diff;
+
+	w->abacus.delay = 0;
+	w->abacus.rightToLeftAdd = True;
+	(void) printf("%s\n", ((w->abacus.rightToLeftAdd) ?
+		"right to left" : "left to right"));
+/*#define FRACT 1.0 
+#define FRACT 0.1*/
+#define FRACT 0.01 
+	for (x = startX; x < finishX; x++) {
+		if (op == 'v' || op == 'u') {
+			(void) sprintf(buffer, "%1.9g%c", FRACT * FRACT * x * x, op);
+			(void) printf("%g: %s = ", FRACT * x, buffer);
+			moves = 0;
+			for (i = 0; i < max; i++) {
+				teachStep(w, buffer, -1);
+				if (w->abacus.step == 0) {
+					break;
+				} else {
+					moves = w->abacus.step;
+				}
+			}
+			trimStringBuffer(w->abacus.rString);
+			trimStringBuffer(w->abacus.bString);
+			ans = atof(w->abacus.bString);
+			rem = atof(w->abacus.rString);
+			diff = ABS(2.0 * x * FRACT - ans);
+			(void) printf("%s / 2 (remainder %s)%s%s%s\n",
+#ifdef LEE_ROOT
+				w->abacus.bString,
+#else
+				w->abacus.sString,
+#endif
+				w->abacus.rString,
+				((w->abacus.step != 0) ? "s" : ""),
+				((rem != 0.0) ? "r" : ""),
+				((diff >= EPSILON) ? "a" : ""));
+			if ((w->abacus.step != 0) ||
+					(rem != 0.0) ||
+					(diff >= EPSILON))
+				exit(-1);
+		} else {
+			for (y = startY; y < finishY; y++) {
+				(void) sprintf(buffer, "%d%c%d", x, op, y);
+				(void) printf("%s = ", buffer);
+				for (i = 0; i < max; i++) {
+					teachStep(w, buffer, -1);
+					if (w->abacus.step == 0)
+						break;
+				}
+				if (op =='/') {
+					(void) printf("%s\n", w->abacus.sString);
+				} else {
+					(void) printf("%s\n", w->abacus.rString);
+				}
+			}
+		}
+	}
+}
+#endif
